require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.provider.ContactsContract"
import "android.net.Uri"
import "android.Manifest"
import "android.content.pm.PackageManager"
import "android.content.Intent"
import "java.util.*"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.os.Build"
import "android.os.VibrationEffect"
import "android.os.Vibrator"

local activity = activity
local contactList = {}
local contactsSpinner = nil
local VERSION = "2.3"

function onCreate(savedInstanceState)
    activity.setTitle("Speed Dial " .. VERSION)
    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    -- Request contacts permissions
    if not checkContactPermissions() then
        requestContactPermissions()
    else
        loadContacts(layout)
        loadManualDialer(layout)
    end

    activity.setContentView(layout)
end

function checkContactPermissions()
    return activity.checkSelfPermission(Manifest.permission.READ_CONTACTS) == PackageManager.PERMISSION_GRANTED
end

function requestContactPermissions()
    activity.requestPermissions({ Manifest.permission.READ_CONTACTS }, 1)
end

function loadContacts(layout)
    -- Create the contact selection dropdown
    contactsSpinner = Spinner(activity)
    layout.addView(contactsSpinner)

    -- Create the call button
    local callButton = Button(activity)
    callButton.setText("Call Contact")
    callButton.setOnClickListener{
        onClick = function(v)
            local selectedPosition = contactsSpinner.getSelectedItemPosition()
            local phoneNumber = contactList[selectedPosition + 1].phoneNumber
            callContact(phoneNumber)
            vibrateDevice()
        end
    }
    layout.addView(callButton)

    -- Create the voice search button
    local voiceSearchButton = Button(activity)
    voiceSearchButton.setText("Voice Search")
    voiceSearchButton.setOnClickListener {
        onClick = function(v)
            startVoiceSearch()
            vibrateDevice()
        end
    }
    layout.addView(voiceSearchButton)

    -- Load contacts
    populateContacts(contactsSpinner)
end

function loadManualDialer(layout)
    -- Create the manual dial AutoCompleteTextView
    local manualDialEditText = AutoCompleteTextView(activity)
    manualDialEditText.setHint("Enter phone number")
    local adapter = ArrayAdapter(activity, android.R.layout.simple_dropdown_item_1line, {})
    manualDialEditText.setAdapter(adapter)
    layout.addView(manualDialEditText)

    -- Create the manual dial button
    local manualDialButton = Button(activity)
    manualDialButton.setText("Call Manual")
    manualDialButton.setOnClickListener{
        onClick = function(v)
            local phoneNumber = manualDialEditText.getText().toString()
            callContact(phoneNumber)
            vibrateDevice()
        end
    }
    layout.addView(manualDialButton)

    -- Populate the contact suggestions
    populateContactSuggestions(adapter)
end

function populateContacts(contactsSpinner)
    local cursor = activity.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, nil, nil, nil, nil)
    local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, {})
    contactsSpinner.setAdapter(adapter)

    while cursor.moveToNext() do
        local id = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
        local name = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
        local phoneNumber = getPhoneNumber(id)

        if phoneNumber then
            adapter.add(name)
            table.insert(contactList, {name = name, phoneNumber = phoneNumber})
        end
    end
    cursor.close()
    adapter.notifyDataSetChanged()
end

function populateContactSuggestions(adapter)
    local cursor = activity.getContentResolver().query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, nil, nil, nil, nil)
    while cursor.moveToNext() do
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        adapter.add(phoneNumber)
    end
    cursor.close()
    adapter.notifyDataSetChanged()
end

function getPhoneNumber(contactId)
    local cursor = activity.getContentResolver().query(ContactsContract.CommonDataKinds.Phone.CONTENT_URI, nil, ContactsContract.CommonDataKinds.Phone.CONTACT_ID .. " = ?", {contactId}, nil)
    if cursor.moveToFirst() then
        local phoneNumber = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER))
        cursor.close()
        return phoneNumber
    end
    cursor.close()
    return nil
end

function callContact(phoneNumber)
    local intent = Intent(Intent.ACTION_CALL)
    intent.setData(Uri.parse("tel:" .. phoneNumber))
    activity.startActivity(intent)
end

function startVoiceSearch()
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(activity)

    local recognitionListener = RecognitionListener {
        onResults = function(results)
            local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if resultArray ~= nil and resultArray.size() > 0 then
                local searchQuery = resultArray.get(0)
                searchContacts(searchQuery)
            end
            destroySpeechRecognizer(speechRecognizer)
            vibrateDevice()
        end,
        onError = function()
            destroySpeechRecognizer(speechRecognizer)
            vibrateDevice()
            return false
        end
    }

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault().getLanguage())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, activity.getPackageName())

    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(recognitionListener)
end

function destroySpeechRecognizer(rec)
    if rec then
        rec.destroy()
        rec = nil
    end
    return true
end

function searchContacts(searchQuery)
    if contactsSpinner ~= nil then
        local cursor = activity.getContentResolver().query(ContactsContract.Contacts.CONTENT_URI, nil, ContactsContract.Contacts.DISPLAY_NAME .. " LIKE ?", {"%" .. searchQuery .. "%"}, nil)
        local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, {})
        contactsSpinner.setAdapter(adapter)

        contactList = {}
        while cursor.moveToNext() do
            local id = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts._ID))
            local name = cursor.getString(cursor.getColumnIndex(ContactsContract.Contacts.DISPLAY_NAME))
            local phoneNumber = getPhoneNumber(id)

            if phoneNumber then
                adapter.add(name)
                table.insert(contactList, {name = name, phoneNumber = phoneNumber})
            end
        end
        cursor.close()
        adapter.notifyDataSetChanged()
    else
        -- Handle the case where contactsSpinner is nil
        -- You can display an error message or perform some other action
    end
end

function vibrateDevice()
    local vibrator = activity.getSystemService(Context.VIBRATOR_SERVICE)
    if Build.VERSION.SDK_INT >= Build.VERSION_CODES.O then
        vibrator.vibrate(VibrationEffect.createOneShot(200, VibrationEffect.DEFAULT_AMPLITUDE))
    else
        vibrator.vibrate(200)
    end
end